package org.processmining.plugins.actitrac;

import java.io.IOException;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.actitrac.algorithm.Clusterer;
import org.processmining.plugins.actitrac.algorithm.GreedyClusterer;
import org.processmining.plugins.actitrac.algorithm.GreedyDistanceClusterer;
import org.processmining.plugins.actitrac.ui.ClustererEventListenerUI;
import org.processmining.plugins.actitrac.ui.ParametersWizard;
import org.processmining.plugins.actitrac.utils.ClustererEventListener;
import org.processmining.plugins.actitrac.utils.OperationCancelledException;


/**
 * The ActiTraC ProM-plugin.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-07-29
 */
public class ActiTraC {

	/**
	 * Calls the ActiTraC-plugin. This will start calculations on the given
	 * XLog.
	 */
	@Plugin(
			name = "ActiTraC",
			parameterLabels = { "Log" },
			returnLabels = { "ActiTraC Clustering" },
			returnTypes = { ActiTraCClustering.class },
			userAccessible = true,
			help = "Activity-based Trace Clustering")
	@UITopiaVariant(
			affiliation = "KULeuven",
			author = "Jochen De Weerdt",
			email = "Jochen.DeWeerdt@econ.kuleuven.be",
			website = "http://econ.kuleuven.be")
	public static ActiTraCClustering main(UIPluginContext context, XLog log) throws IOException {
		try {
			// Show the user a wizard
			ParametersWizard conf = new ParametersWizard(context);
			conf.show();

			// Create a new Clusterer
			Clusterer clusterer;

			// Assign a ClustererEventListenerUI-object that is linked to the
			// current context to the Clusterer
			ClustererEventListener eventListener = new ClustererEventListenerUI(
					context, log, conf.getATCParameters());

			if (conf.getATCParameters().isGreedy())
				clusterer = new GreedyClusterer(log, conf.getATCParameters(),
						conf.getHeuristicsMinerSettings(), eventListener);
			else
				clusterer = new GreedyDistanceClusterer(log,
						conf.getATCParameters(),
						conf.getHeuristicsMinerSettings(), eventListener);

			// Start calculations
			clusterer.cluster();

			// Return the result
			return clusterer.getClustering();
		} catch (OperationCancelledException e) {
			// The user cancelled any operation, notify ProM that no result will
			// be given and stop
			context.getFutureResult(0).cancel(true);
			return null;
		}
	}

}
