package org.processmining.plugins.actitrac;

import org.processmining.plugins.actitrac.types.ClusterTree;
import org.processmining.plugins.actitrac.types.GroupedXLog;
import org.processmining.plugins.actitrac.utils.ATCParameters;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.settings.HeuristicsMinerSettings;


/**
 * This class represents a certain GroupedXLog, which has been divided into an
 * arbitrary number of clusters. This clustering has been done according to
 * ATCParameters and HeuristicsMinerSettings, whom are also stored along with
 * the clustering itself.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-07-29
 */
public class ActiTraCClustering {

	/**
	 * The root of the ClusterTree in which all Clusters are located.
	 */
	private ClusterTree tree;

	/**
	 * The ATCParameters used to calculate this ActiTraCClustering.
	 */
	private ATCParameters parameters;

	/**
	 * The original log.
	 */
	private GroupedXLog log;

	/**
	 * The HeuristicsMinerSettings used to calculate this ActiTraCClustering.
	 */
	private HeuristicsMinerSettings settings;

	/**
	 * Create a new ActiTraCClustering with the given log, tree, parameters and
	 * settings.
	 * 
	 * @param log
	 *            The GroupedXLog whose clustering is being represented.
	 * @param tree
	 *            The ClusterTree that represents the clustering of the log.
	 * @param parameters
	 *            The ActiTraCParameters used to do the clustering of the log.
	 * @param settings
	 *            The HeuristicsMinerSettings used to do the clustering of the
	 *            log.
	 */
	public ActiTraCClustering(GroupedXLog log, ClusterTree tree,
			ATCParameters parameters, HeuristicsMinerSettings settings) {
		this.log = log;
		this.tree = tree;
		this.parameters = parameters;
		this.settings = settings;
	}

	/**
	 * Return a copy of the represented GroupedXLog.
	 */
	public GroupedXLog getLog() {
		return (GroupedXLog) log.clone();
	}

	/**
	 * Return the used ATCParameters.
	 */
	public ATCParameters getParameters() {
		return parameters;
	}

	/**
	 * Return the used HeuristicsMinerSettings.
	 */
	public HeuristicsMinerSettings getSettings() {
		return settings;
	}

	/**
	 * Return the ClusterTreeRoot that represents the clustering of the original
	 * log. This is the original object, thus altering it will change the
	 * Clustering object.
	 */
	public ClusterTree getTree() {
		return tree;
	}
}
