package org.processmining.plugins.actitrac.types;

/**
 * This class describes the root node of a ClusterTree. This node differs from a
 * ClusterTreeInnerNode by being associated with a Cluster. This cluster is not
 * part of the output of the ActiTraC algorithm. Instead, it is a slightly
 * processed variant of the input for the algorithm.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-07-29
 */
public class ClusterTreeRoot extends ClusterTreeInnerNode {

	/**
	 * The cluster describing the algorithm input.
	 */
	private Cluster pseudoCluster;

	/**
	 * Create a new ClusterTreeRoot. The given Cluster will be seen as the input
	 * that has been used to build the remainder of the tree.
	 */
	public ClusterTreeRoot(Cluster pseudoCluster) {
		this.pseudoCluster = pseudoCluster;
	}

	/**
	 * Get the pseudo-cluster for this node. This cluster is equivalent to the
	 * input for the ActiTraC-algorithm.
	 */
	public Cluster getPseudoCluster() {
		return pseudoCluster;
	}
}
