package org.processmining.plugins.actitrac.types;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.guidetreeminer.distance.EuclideanDistance;
import org.processmining.plugins.guidetreeminer.encoding.ActivityOverFlowException;
import org.processmining.plugins.guidetreeminer.encoding.EncodeActivitySet;
import org.processmining.plugins.guidetreeminer.encoding.EncodeTraces;
import org.processmining.plugins.guidetreeminer.encoding.EncodingNotFoundException;
import org.processmining.plugins.guidetreeminer.featureextraction.FeatureExtraction;
import org.processmining.plugins.guidetreeminer.featureextraction.FeatureMatrix;
import org.processmining.plugins.guidetreeminer.featureextraction.FeatureSelection;
import org.processmining.plugins.guidetreeminer.similarity.FScoreSimilarity;
import org.processmining.plugins.guidetreeminer.types.DistanceMetricType;
import org.processmining.plugins.guidetreeminer.types.GTMFeature;
import org.processmining.plugins.guidetreeminer.types.GTMFeatureType;
import org.processmining.plugins.guidetreeminer.types.Normalization;
import org.processmining.plugins.guidetreeminer.types.SimilarityDistanceMetricType;
import org.processmining.plugins.guidetreeminer.util.EquivalenceClass;


public class MaxRepeatAlphabetDistanceXLog extends DistanceXLog {


	//Encoding of the log: required variables
	int encodingLength, averageTraceLength;

	Map<String, String> charActivityMap, activityCharMap;
	List<String> encodedTraceList, uniqueEncodedTraceList;
	Set<String> duplicateTraceSet;
	Map<String, TreeSet<Integer>> encodedTraceIdenticalIndicesMap;

	Map<String, Integer> encodedActivityCountMap, encodedActivityUniqueTraceCountMap;
	
	//Computing feature set: required variables
	Map<GTMFeature, Set<String>> featureOriginalSequenceFeatureSetMap;
	Map<GTMFeature, Set<String>> featureBaseSequenceFeatureSetMap;
	Map<GTMFeature, Set<String>> featureActualSequenceFeatureSetMap = new HashMap<GTMFeature, Set<String>>();

	Map<GTMFeature, Map<Set<String>, Set<String>>> featureOriginalAlphabetFeatureSetMap = new HashMap<GTMFeature, Map<Set<String>, Set<String>>>();
	Map<GTMFeature, Map<Set<String>, Set<String>>> featureBaseAlphabetFeatureSetMap = new HashMap<GTMFeature, Map<Set<String>, Set<String>>>();
	Map<GTMFeature, Map<Set<String>, Set<String>>> featureActualAlphabetFeatureSetMap = new HashMap<GTMFeature, Map<Set<String>, Set<String>>>();
	Set<String> unionSequenceFeatureSet;
	Set<String> filteredSequenceFeatureSet;

	Map<String, Integer> actualNonOverlapSequenceCountMap = new HashMap<String, Integer>();
	Map<String, Integer> actualNonOverlapSequenceInstanceCountMap = new HashMap<String, Integer>();

	Map<Set<String>, Integer> actualNonOverlapAlphabetCountMap = new HashMap<Set<String>, Integer>();
	Map<Set<String>, Integer> actualNonOverlapAlphabetInstanceCountMap = new HashMap<Set<String>, Integer>();
	Map<Set<String>, Set<String>> actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap = new HashMap<Set<String>, Set<String>>();

	int MAX_STRLENGTH = 5000;
	boolean isSimilarity = false;

	
	
	public MaxRepeatAlphabetDistanceXLog(GroupedXLog glog) throws IOException {
		super(glog);
		XLog log = glog.getLog();
		
		encodeLog(log);
		computeFeatureSets(log);
		setActualAlphabetPattern();
		
		if (featureActualAlphabetFeatureSetMap.size() > 1)
			findNonOverlapAlphabetPatternCountOptimized(featureActualAlphabetFeatureSetMap
					.get(GTMFeature.MIXA));
//		else
//			findNonOverlapAlphabetPatternCountOptimized(featureActualAlphabetFeatureSetMap
//					.get(input.getSelectedFeatureSet().toArray()[0]));


		float[][] OriginalDistanceMatrix = computeSimilarityDistanceMatrix();
		
		printMatrixToText(OriginalDistanceMatrix);
		
		int calculations = 0;
		distance = new float[glog.size()][];
		for (int i = 0; i < glog.size(); i++) {
			distance[i] = new float[glog.size() - i];
			for (int j = i + 1; j < glog.size(); j++) {
				distance[i][j - i] = OriginalDistanceMatrix[i][j - i];
				calculations++;
			}
		}
		
		
	}
	
	private void encodeLog(XLog log) throws IOException {
		/*
		 * traceIndex is used to keep track of the number of traces;
		 * totalNoEvents counts the number of events/activities in the entire
		 * event log. activitySet accumulates the set of distinct
		 * activities/events in the event log; it doesn't store the trace
		 * identifier for encoding; Encoding trace identifier is required only
		 * when any of the maximal repeat (alphabet) features is selected
		 */


		
		int traceIndex = 0;
		int totalNoEvents = 0;
		Set<String> activitySet = new HashSet<String>();
		XAttributeMap attributeMap;
		Set<String> eventTypeSet = new HashSet<String>();

		for (XTrace trace : log) {
			totalNoEvents += trace.size();
			for (XEvent event : trace) {
				attributeMap = event.getAttributes();
				activitySet.add(attributeMap.get("concept:name").toString()
						+ "-"
						+ attributeMap.get("lifecycle:transition").toString());
				eventTypeSet.add(attributeMap.get("lifecycle:transition")
						.toString());
			}
		}

		averageTraceLength = totalNoEvents / log.size();
		encodedTraceIdenticalIndicesMap = new HashMap<String, TreeSet<Integer>>();
		
	    try {
			EncodeActivitySet encodeActivitySet = new EncodeActivitySet(
					activitySet);
			encodingLength = encodeActivitySet.getEncodingLength();

			activityCharMap = encodeActivitySet.getActivityCharMap();
			charActivityMap = encodeActivitySet.getCharActivityMap();
			/*
			 * Encode each trace to a charStream
			 */
			
			
			EncodeTraces encodeTraces = new EncodeTraces(activityCharMap, log);
			encodedTraceList = encodeTraces.getCharStreamList();

			uniqueEncodedTraceList = new ArrayList<String>();

			traceIndex = 0;
			TreeSet<Integer> encodedTraceIdenticalIndicesSet;
			for (String encodedTrace : encodedTraceList) {
				if (encodedTraceIdenticalIndicesMap.containsKey(encodedTrace)) {
					encodedTraceIdenticalIndicesSet = encodedTraceIdenticalIndicesMap
							.get(encodedTrace);
				} else {
					encodedTraceIdenticalIndicesSet = new TreeSet<Integer>();
					uniqueEncodedTraceList.add(encodedTrace);
				}
				encodedTraceIdenticalIndicesSet.add(traceIndex);
				encodedTraceIdenticalIndicesMap.put(encodedTrace,
						encodedTraceIdenticalIndicesSet);

				traceIndex++;
			}
			duplicateTraceSet = new HashSet<String>();
			for (String encodedTrace : encodedTraceIdenticalIndicesMap.keySet()) {
				if (encodedTraceIdenticalIndicesMap.get(encodedTrace).size() > 1)
					duplicateTraceSet.add(encodedTrace);
			}

			encodedActivityCountMap = new HashMap<String, Integer>();
			encodedActivityUniqueTraceCountMap = new HashMap<String, Integer>();
			int traceLength, noIdenticalTraces, count;
			String encodedActivity;

			for (String encodedTrace : uniqueEncodedTraceList) {
				noIdenticalTraces = encodedTraceIdenticalIndicesMap.get(
						encodedTrace).size();
				traceLength = encodedTrace.length() / encodingLength;

				for (int i = 0; i < traceLength; i++) {
					encodedActivity = encodedTrace.substring(
							i * encodingLength, (i + 1) * encodingLength);
					count = noIdenticalTraces;
					if (encodedActivityCountMap.containsKey(encodedActivity)) {
						count += encodedActivityCountMap.get(encodedActivity);
					}
					encodedActivityCountMap.put(encodedActivity, count);

					count = 1;
					if (encodedActivityUniqueTraceCountMap
							.containsKey(encodedActivity)) {
						count += encodedActivityUniqueTraceCountMap
								.get(encodedActivity);
					}
					encodedActivityUniqueTraceCountMap.put(encodedActivity,
							count);
				}
			}
			// printActivityList(uniqueEncodedTraceList,encodedTraceIdenticalIndicesMap);
		} catch (ActivityOverFlowException e) {
			e.printStackTrace();
		} catch (EncodingNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	
	
	private void computeFeatureSets(XLog log) {
		if (featureOriginalSequenceFeatureSetMap == null)
			featureOriginalSequenceFeatureSetMap = new HashMap<GTMFeature, Set<String>>();
		if (featureBaseSequenceFeatureSetMap == null)
			featureBaseSequenceFeatureSetMap = new HashMap<GTMFeature, Set<String>>();
		if (featureOriginalAlphabetFeatureSetMap == null)
			featureOriginalAlphabetFeatureSetMap = new HashMap<GTMFeature, Map<Set<String>, Set<String>>>();
		if (featureBaseAlphabetFeatureSetMap == null)
			featureBaseAlphabetFeatureSetMap = new HashMap<GTMFeature, Map<Set<String>, Set<String>>>();

		featureOriginalSequenceFeatureSetMap.clear();
		featureBaseSequenceFeatureSetMap.clear();

		featureOriginalAlphabetFeatureSetMap.clear();
		featureBaseAlphabetFeatureSetMap.clear();

		//Set<GTMFeature> selectedFeatureSet = input.getSelectedFeatureSet();
		Set<GTMFeature> selectedFeatureSet = new HashSet<GTMFeature>();
		
		//selectedFeatureSet.add(GTMFeature.TR);
		selectedFeatureSet.add(GTMFeature.MRA);
		
		
		List<GTMFeature> repeatFeatureTypeList = new ArrayList<GTMFeature>();
		List<GTMFeature> repeatAlphabetFeatureTypeList = new ArrayList<GTMFeature>();
		Map<Set<String>, Set<String>> alphabetFeatureSetMap;
		FeatureSelection featureSelection;
		

		for (GTMFeature feature : selectedFeatureSet) {
			switch (feature) {
			case IE:
				featureOriginalSequenceFeatureSetMap.put(feature,
						charActivityMap.keySet());
				featureBaseSequenceFeatureSetMap.put(feature, charActivityMap
						.keySet());
				alphabetFeatureSetMap = (new FeatureExtraction(
						encodingLength, encodedTraceList, activityCharMap,
						feature)).getAlphabetFeatureSetMap();
				featureOriginalAlphabetFeatureSetMap.put(feature,
						alphabetFeatureSetMap);
				featureSelection = new FeatureSelection(encodingLength);
				featureBaseAlphabetFeatureSetMap
				.put(feature,featureSelection.getBaseFeatureAlphabetFeatureSetMap(alphabetFeatureSetMap));
				break;
//			case KGram:
//				Set<String> kGramFeatureSet = new FeatureExtraction(
//						encodingLength, encodedTraceList, feature, input
//								.getKGramValue()).getSequenceFeatureSet();
//				featureOriginalSequenceFeatureSetMap.put(feature,
//						kGramFeatureSet);
//				featureBaseSequenceFeatureSetMap.put(feature, kGramFeatureSet);
//				break;
			case TR:
				Set<String> tandemRepeatFeatureSet = (new FeatureExtraction(
						encodingLength, encodedTraceList, activityCharMap,
						feature)).getSequenceFeatureSet();
				featureOriginalSequenceFeatureSetMap.put(feature,
						tandemRepeatFeatureSet);
				featureSelection = new FeatureSelection(
						encodingLength);
				featureBaseSequenceFeatureSetMap.put(feature, featureSelection
						.getBaseFeatureSet(tandemRepeatFeatureSet));
				break;
			case MR:
			case SMR:
			case NSMR:
				repeatFeatureTypeList.add(feature);
				break;
			case TRA:
				alphabetFeatureSetMap = (new FeatureExtraction(
						encodingLength, encodedTraceList, activityCharMap,
						feature)).getAlphabetFeatureSetMap();
				featureSelection = new FeatureSelection(encodingLength);
				featureOriginalAlphabetFeatureSetMap.put(feature,
						alphabetFeatureSetMap);
				featureBaseAlphabetFeatureSetMap
						.put(
								feature,
								featureSelection
										.getBaseFeatureAlphabetFeatureSetMap(alphabetFeatureSetMap));
				break;
			case MRA:
			case SMRA:
			case NSMRA:
				repeatAlphabetFeatureTypeList.add(feature);
				break;
			default:
				break;
			}
		}

		if ((repeatFeatureTypeList.size() > 0)
				|| (repeatAlphabetFeatureTypeList.size() > 0)) {
			/*
			 * Create a new encoded trace set so that the trace indices are
			 * considered for encoding; This would be used as a distinct
			 * delimiter for combining the streams
			 */

			int traceIndex = 0, totalNoEvents = 0;
			int encodingLength = 1;
			Set<String> activitySet = new HashSet<String>();
			XAttributeMap attributeMap;
			Map<String, String> extendedActivityCharMap = null;
			Map<String, String> extendedCharActivityMap = null;

			StringBuilder combinedStringBuilder = new StringBuilder();
			for (XTrace trace : log) {
				totalNoEvents += trace.size();
				for (XEvent event : trace) {
					attributeMap = event.getAttributes();
					activitySet.add(attributeMap.get("concept:name").toString()
							+ "-"
							+ attributeMap.get("lifecycle:transition")
									.toString());
				}
				activitySet.add("Trace" + traceIndex);
				traceIndex++;
			}

			try {
				EncodeActivitySet encodeActivitySet = new EncodeActivitySet(
						activitySet);
				encodingLength = encodeActivitySet.getEncodingLength();

				extendedActivityCharMap = encodeActivitySet
						.getActivityCharMap();
				extendedCharActivityMap = encodeActivitySet
						.getCharActivityMap();

				/*
				 * Encode each trace to a charStream
				 */
				EncodeTraces encodeTraces = new EncodeTraces(
						extendedActivityCharMap, log);
				List<String> extendedEncodedTraceList = encodeTraces
						.getCharStreamList();

				Set<String> extendedEncodedTraceSet = new HashSet<String>();
				extendedEncodedTraceSet.addAll(extendedEncodedTraceList);

				traceIndex = 0;
				for (String charStream : extendedEncodedTraceSet) {
					combinedStringBuilder.append(charStream);
					combinedStringBuilder.append(extendedActivityCharMap
							.get("Trace" + traceIndex));
					traceIndex++;
				}
			} catch (ActivityOverFlowException e) {
				//context.log("Error while encoding Log: "+e.getLocalizedMessage());
			} catch (EncodingNotFoundException e) {
				//context.log("Error while encoding Log: "+e.getLocalizedMessage());
			}

			FeatureExtraction featureExtraction = new FeatureExtraction();
			if (repeatFeatureTypeList.size() > 0) {
				Set<String> repeatFeatureSet;
				featureSelection = new FeatureSelection(
						encodingLength);
				Map<GTMFeature, Set<String>> repeatFeatureFeatureSetMap = featureExtraction
						.getRepeatFeatureFeatureSetMap(encodingLength,
								combinedStringBuilder.toString(),
								repeatFeatureTypeList);
				for (GTMFeature feature : repeatFeatureFeatureSetMap.keySet()) {
					repeatFeatureSet = getOriginalEncodedFeatureSet(
							encodingLength, repeatFeatureFeatureSetMap
									.get(feature), extendedCharActivityMap);
					featureOriginalSequenceFeatureSetMap.put(feature,
							repeatFeatureSet);
					featureBaseSequenceFeatureSetMap.put(feature,
							featureSelection
									.getBaseFeatureSet(repeatFeatureSet));
				}
			} else if (repeatAlphabetFeatureTypeList.size() > 0) {
				Map<Set<String>, Set<String>> alphabetFeatureFeatureSetMap;
				featureSelection = new FeatureSelection(
						encodingLength);
				Map<GTMFeature, Map<Set<String>, Set<String>>> repeatAlphabetFeatureFeatureSetMap = featureExtraction
						.getRepeatAlphabetFeatureFeatureSetMap(encodingLength,
								combinedStringBuilder.toString(),
								repeatAlphabetFeatureTypeList);
				for (GTMFeature feature : repeatAlphabetFeatureFeatureSetMap
						.keySet()) {
					alphabetFeatureFeatureSetMap = getOriginalEncodedAlphabetFeatureSetMap(
							encodingLength, repeatAlphabetFeatureFeatureSetMap
									.get(feature), extendedCharActivityMap);
					featureOriginalAlphabetFeatureSetMap.put(feature,
							alphabetFeatureFeatureSetMap);
					featureBaseAlphabetFeatureSetMap
							.put(
									feature,
									featureSelection
											.getBaseFeatureAlphabetFeatureSetMap(alphabetFeatureFeatureSetMap));
				}
			}

		}

		// printFeatureSets();

		computeUnionFeatureSet();
	}
	
	private Set<String> getOriginalEncodedFeatureSet(
			int extendedEncodingLength, Set<String> featureSet,
			Map<String, String> extendedCharActivityMap) {
		Set<String> originalEncodedFeatureSet = new HashSet<String>();

		int featureLength;
		String symbol, originalEncodedFeature;
		for (String feature : featureSet) {
			featureLength = feature.length() / extendedEncodingLength;
			originalEncodedFeature = "";
			for (int i = 0; i < featureLength; i++) {
				symbol = feature.substring(i * extendedEncodingLength, (i + 1)
						* extendedEncodingLength);
				originalEncodedFeature += activityCharMap
						.get(extendedCharActivityMap.get(symbol));
			}
			originalEncodedFeatureSet.add(originalEncodedFeature);
		}

		return originalEncodedFeatureSet;
	}

	private Map<Set<String>, Set<String>> getOriginalEncodedAlphabetFeatureSetMap(
			int extendedEncodingLength,
			Map<Set<String>, Set<String>> alphabetFeatureSetMap,
			Map<String, String> extendedCharActivityMap) {
		Set<String> unionRepeatSet = new HashSet<String>();
		Set<String> repeatAlphabetEquivalenceClassSet;
		int repeatLength;
		String originalEncodedRepeat;
		for (Set<String> repeatAlphabet : alphabetFeatureSetMap.keySet()) {
			repeatAlphabetEquivalenceClassSet = alphabetFeatureSetMap
					.get(repeatAlphabet);
			for (String repeat : repeatAlphabetEquivalenceClassSet) {
				repeatLength = repeat.length() / extendedEncodingLength;
				originalEncodedRepeat = "";
				for (int i = 0; i < repeatLength; i++) {
					originalEncodedRepeat += activityCharMap
							.get(extendedCharActivityMap.get(repeat.substring(i
									* extendedEncodingLength, (i + 1)
									* extendedEncodingLength)));
				}
				unionRepeatSet.add(originalEncodedRepeat);
			}
		}

		EquivalenceClass equivalenceClass = new EquivalenceClass();
		return equivalenceClass.getAlphabetEquivalenceClassMap(encodingLength,
				unionRepeatSet);
	}
	
	
	private void computeUnionFeatureSet() {
		//context.log("Computing union feature set");
		//Set<GTMFeature> selectedFeatureSet = input.getSelectedFeatureSet();
		
		
		HashSet<GTMFeature> selectedFeatureSet = new HashSet<GTMFeature>();
		selectedFeatureSet.add(GTMFeature.TR);
		selectedFeatureSet.add(GTMFeature.MRA);

		GTMFeatureType GTMFeatureType1 = GTMFeatureType.Alphabet;
		
		

		if (selectedFeatureSet.size() > 1) {
			//if (input.getFeatureType().equals(GTMFeatureType.Sequence)) {
			if (GTMFeatureType1 == GTMFeatureType.Sequence) {
				Set<String> unionSequenceFeatureSet = new HashSet<String>();
				for (GTMFeature feature : featureOriginalSequenceFeatureSetMap
						.keySet()) {
					unionSequenceFeatureSet
							.addAll(featureOriginalSequenceFeatureSetMap
									.get(feature));
				}
				featureOriginalSequenceFeatureSetMap.put(GTMFeature.MIX,
						unionSequenceFeatureSet);

				unionSequenceFeatureSet = new HashSet<String>();
				for (GTMFeature feature : featureBaseSequenceFeatureSetMap
						.keySet()) {
					unionSequenceFeatureSet
							.addAll(featureBaseSequenceFeatureSetMap
									.get(feature));
				}
				featureBaseSequenceFeatureSetMap.put(GTMFeature.MIX,
						unionSequenceFeatureSet);
			//} else if (input.getFeatureType().equals(GTMFeatureType.Alphabet)) {
			} else if (GTMFeatureType1 == GTMFeatureType.Alphabet) {
				Map<Set<String>, Set<String>> unionAlphabetFeatureSetMap = new HashMap<Set<String>, Set<String>>();
				Map<Set<String>, Set<String>> alphabetFeatureSetMap;
				Set<String> alphabetEquivalenceClassFeatureSet;
				for (GTMFeature feature : featureOriginalAlphabetFeatureSetMap
						.keySet()) {
					alphabetFeatureSetMap = featureOriginalAlphabetFeatureSetMap
							.get(feature);
					for (Set<String> alphabet : alphabetFeatureSetMap.keySet()) {
						if (unionAlphabetFeatureSetMap.containsKey(alphabet)) {
							alphabetEquivalenceClassFeatureSet = unionAlphabetFeatureSetMap
									.get(alphabet);
						} else {
							alphabetEquivalenceClassFeatureSet = new TreeSet<String>();
						}
						alphabetEquivalenceClassFeatureSet
								.addAll(alphabetFeatureSetMap.get(alphabet));
						unionAlphabetFeatureSetMap.put(alphabet,
								alphabetEquivalenceClassFeatureSet);
					}
				}

				featureOriginalAlphabetFeatureSetMap.put(GTMFeature.MIXA,
						unionAlphabetFeatureSetMap);

				unionAlphabetFeatureSetMap = new HashMap<Set<String>, Set<String>>();
				for (GTMFeature feature : featureBaseAlphabetFeatureSetMap
						.keySet()) {
					alphabetFeatureSetMap = featureBaseAlphabetFeatureSetMap
							.get(feature);
					for (Set<String> alphabet : alphabetFeatureSetMap.keySet()) {
						if (unionAlphabetFeatureSetMap.containsKey(alphabet)) {
							alphabetEquivalenceClassFeatureSet = unionAlphabetFeatureSetMap
									.get(alphabet);
						} else {
							alphabetEquivalenceClassFeatureSet = new TreeSet<String>();
						}
						alphabetEquivalenceClassFeatureSet
								.addAll(alphabetFeatureSetMap.get(alphabet));
						unionAlphabetFeatureSetMap.put(alphabet,
								alphabetEquivalenceClassFeatureSet);
					}
				}
				featureBaseAlphabetFeatureSetMap.put(GTMFeature.MIXA,
						unionAlphabetFeatureSetMap);
			}
		}
		//context.log("Union feature set computed");
	}

	private void setActualAlphabetPattern() {
		/*
		 * Set the actualPattern; If baseFeatureCheckBox is selected, then
		 * consider baseFeatureSet else, consider original
		 */
		featureActualAlphabetFeatureSetMap.clear();
		//if (input.isBaseFeatures) {
		if (true) {
			for (GTMFeature feature : featureBaseAlphabetFeatureSetMap.keySet()) {
				featureActualAlphabetFeatureSetMap.put(feature,
						featureBaseAlphabetFeatureSetMap.get(feature));
			}
		} 
//		else {
//			for (GTMFeature feature : featureOriginalAlphabetFeatureSetMap
//					.keySet()) {
//				featureActualAlphabetFeatureSetMap.put(feature,
//						featureOriginalAlphabetFeatureSetMap.get(feature));
//			}
//		}
	}
	
	
	private void findNonOverlapAlphabetPatternCountOptimized(
			Map<Set<String>, Set<String>> alphabetFeatureSetMap) {
//		System.out.println("Here: find NonOverlapAlphabetPatternCount: "+alphabetFeatureSetMap.keySet());
		actualNonOverlapAlphabetCountMap.clear();
		actualNonOverlapAlphabetInstanceCountMap.clear();
		actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.clear();
		
		Set<String> alphabetEquivalenceClassFeatureSet;
		boolean currentTraceHasPattern;
		int encodedTraceLength;
		int maxCount, patternCount, repeatLength, noMatches, noIdenticalTraces, count;
		String maxCountPattern;
		Pattern pattern;
		Matcher matcher;
		Set<Set<String>> encodedTraceContributingAlphabetSet = new HashSet<Set<String>>();

		/*
		 * For each alphabet, check first whether this sub-trace
		 * contains the repeat under the alphabet; if so, get the
		 * non-overlapping count of that repeat alphabet
		 */
		for(String encodedTrace : uniqueEncodedTraceList){
			encodedTraceLength = encodedTrace.length()/encodingLength;
			noIdenticalTraces = encodedTraceIdenticalIndicesMap.get(encodedTrace).size();
			
			encodedTraceContributingAlphabetSet.clear();
			for (Set<String> alphabet : alphabetFeatureSetMap.keySet()) {
				alphabetEquivalenceClassFeatureSet = alphabetFeatureSetMap.get(alphabet);
				currentTraceHasPattern = false;
				for (String repeatPattern : alphabetEquivalenceClassFeatureSet) {
					if (encodedTrace.contains(repeatPattern)) {
						currentTraceHasPattern = true;
						break;
					}
				}
				
				if (currentTraceHasPattern) {
					for (int i = 0; i < encodedTraceLength; i++) {
						maxCount = 0;
						maxCountPattern = "";
						for (String repeatPattern : alphabetEquivalenceClassFeatureSet) {
							/*
							 * First check if this repeat pattern exists
							 * starting at this index; only if it exists
							 * then use the pattern matcher
							 */
							if(encodedTrace.indexOf(repeatPattern, i*encodingLength) == i*encodingLength){
								pattern = Pattern.compile("(" + repeatPattern + "){1,}");
								matcher = pattern.matcher(encodedTrace);
								if (matcher.find(i * encodingLength) && matcher.start() == i * encodingLength) {
									repeatLength = repeatPattern.length() / encodingLength;
									noMatches = (matcher.end() - matcher.start()) / (repeatLength * encodingLength);
									if (noMatches > maxCount) {
										maxCount = noMatches;
										maxCountPattern = repeatPattern;
									}
								}
							}
						}
						if (maxCount > 0) {
							/*
							 * No need to actually compute the counts again
							 * as we have already stored the maxCount and
							 * maxCountPattern information
							 */
							repeatLength = maxCountPattern.length() / encodingLength;
							
							maxCount *= noIdenticalTraces;
							
							i += repeatLength * maxCount - 1;
	
							patternCount = 0;
							if (actualNonOverlapAlphabetCountMap.containsKey(alphabet)) {
								patternCount = actualNonOverlapAlphabetCountMap.get(alphabet);
							}
							actualNonOverlapAlphabetCountMap.put((TreeSet<String>) alphabet, maxCount + patternCount);
							
							encodedTraceContributingAlphabetSet.add((TreeSet<String>)alphabet);
						}
					}
				}
			}

			for(Set<String> alp : encodedTraceContributingAlphabetSet){
				count = 0;
				if(actualNonOverlapAlphabetInstanceCountMap.containsKey(alp)){
					count = actualNonOverlapAlphabetInstanceCountMap.get(alp);
				}
				
				actualNonOverlapAlphabetInstanceCountMap.put(alp, count+noIdenticalTraces);
				if(!actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.containsKey(alp)){
					actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.put(alp, alphabetFeatureSetMap.get(alp));
				}
			}
		}
		
//		System.out.println("Done");
		
	}
	
	private float[][] computeSimilarityDistanceMatrix() {
		GTMFeatureType GTMFeatureType1 = GTMFeatureType.Alphabet;
		SimilarityDistanceMetricType SimilarityDistanceMetricType1 = SimilarityDistanceMetricType.Distance; 
		DistanceMetricType DistanceMetricType1 = DistanceMetricType.Euclidean;
		
		//if (input.getFeatureType().equals(GTMFeatureType.Sequence)) {
		if (GTMFeatureType1 == GTMFeatureType.Sequence) {
			Set<String> filteredSequenceFeatureSet = getFilteredSequenceFeatures();
			FeatureMatrix featureMatrix = new FeatureMatrix(encodingLength,
					uniqueEncodedTraceList, filteredSequenceFeatureSet);
			//if (input.getSimilarityDistanceMetricType() == SimilarityDistanceMetricType.Similarity) {
			if (SimilarityDistanceMetricType1 == SimilarityDistanceMetricType.Similarity) {
				isSimilarity = true;
				return new FScoreSimilarity(encodingLength).getFScore(featureMatrix.getFeatureMatrix());
			//} else if (input.getSimilarityDistanceMetricType() == SimilarityDistanceMetricType.Distance) {
			} else if (SimilarityDistanceMetricType1 == SimilarityDistanceMetricType.Distance) {
				isSimilarity = false;
				//if (input.getDistanceMetricType() == DistanceMetricType.Euclidean) {
				if (DistanceMetricType1 == DistanceMetricType.Euclidean) {
					EuclideanDistance ed = new EuclideanDistance(featureMatrix
							.getFeatureMatrix(), Normalization.NONE);
					return ed.getDistanceMatrix();
				}
			}
		} 

		//else if (input.getFeatureType().equals(GTMFeatureType.Alphabet)) {
		else if (GTMFeatureType1 == GTMFeatureType.Alphabet) {
			Set<Set<String>> filteredAlphabetFeatureSet = getFilteredAlphabetFeatures();
			Map<Set<String>, Set<String>> filteredAlphabetFeatureSetMap = new HashMap<Set<String>, Set<String>>();
			for (Set<String> filteredAlphabet : filteredAlphabetFeatureSet) {
				filteredAlphabetFeatureSetMap.put(filteredAlphabet,
						actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap
								.get(filteredAlphabet));
			}
			FeatureMatrix featureMatrix = new FeatureMatrix(
					encodingLength, uniqueEncodedTraceList,
					filteredAlphabetFeatureSetMap);
			
			//if (input.getSimilarityDistanceMetricType() == SimilarityDistanceMetricType.Similarity) {
			if (SimilarityDistanceMetricType1 == SimilarityDistanceMetricType.Similarity) {
				isSimilarity = true;
				return new FScoreSimilarity(encodingLength).getFScore(featureMatrix.getFeatureMatrix());
			//} else if (input.getSimilarityDistanceMetricType() == SimilarityDistanceMetricType.Distance) {
			} else if (SimilarityDistanceMetricType1 == SimilarityDistanceMetricType.Distance) {
				isSimilarity = false;
				//if (input.getDistanceMetricType() == DistanceMetricType.Euclidean) {
				if (DistanceMetricType1 == DistanceMetricType.Euclidean) {
					EuclideanDistance ed = new EuclideanDistance(featureMatrix
							.getFeatureMatrix(), Normalization.NONE);
					return ed.getDistanceMatrix();
				}
			}
		}
		return null;
	}
	
	private Set<String> getFilteredSequenceFeatures() {
//		int minFrequencyThreshold = input.getMinFrequencyCountThreshold();
//		int minInstanceCountPercentageThreshold = input
//				.getMinInstancePercentageCountThreshold();
		
		int minFrequencyThreshold = 0;
		int minInstanceCountPercentageThreshold = 0;
		
		Set<String> filteredSequenceFeatureSet = new HashSet<String>();
		filteredSequenceFeatureSet.addAll(actualNonOverlapSequenceCountMap
				.keySet());
		for (String sequenceFeature : actualNonOverlapSequenceCountMap.keySet()) {
			if (actualNonOverlapSequenceCountMap.get(sequenceFeature) < minFrequencyThreshold
					|| actualNonOverlapSequenceInstanceCountMap
							.get(sequenceFeature) < minInstanceCountPercentageThreshold)
				filteredSequenceFeatureSet.remove(sequenceFeature);
		}
		return filteredSequenceFeatureSet;
	}

	
	private Set<Set<String>> getFilteredAlphabetFeatures() {
		
//		int minFrequencyThreshold = input.getMinFrequencyCountThreshold();
//		int minInstanceCountPercentageThreshold = input
//				.getMinInstancePercentageCountThreshold();
//		int minAlphabetSize = input.getMinAlphabetSizeThreshold();
//		int maxAlphabetSize = input.getMaxAlphabetSizeThreshold();
		
		int minFrequencyThreshold = 0;
		int minInstanceCountPercentageThreshold = 0;
		int minAlphabetSize = 0;
		int maxAlphabetSize = 10;
		
		
		
		Set<Set<String>> filteredAlphabetFeatureSet = new HashSet<Set<String>>();
		filteredAlphabetFeatureSet.addAll(actualNonOverlapAlphabetCountMap
				.keySet());
		for (Set<String> alphabetFeature : actualNonOverlapAlphabetCountMap
				.keySet()) {
			if (actualNonOverlapAlphabetCountMap.get(alphabetFeature) < minFrequencyThreshold
					|| actualNonOverlapAlphabetInstanceCountMap
							.get(alphabetFeature) < minInstanceCountPercentageThreshold
					|| alphabetFeature.size() < minAlphabetSize
					|| alphabetFeature.size() > maxAlphabetSize)
				filteredAlphabetFeatureSet.remove(alphabetFeature);
		}
		return filteredAlphabetFeatureSet;
	}
	
	protected void printMatrixToText(float[][] similarityDistanceMatrix2) throws IOException{
		

		File yourFile = new File("SimilarityDistanceMatrix.txt");
		yourFile.delete();
//		File yourNewFile = new File("SimilarityDistanceMatrix.txt");
//		yourNewFile.createNewFile();
//		yourNewFile.close();
		
		
		PrintWriter outFile;
		outFile = new PrintWriter(new FileWriter("SimilarityDistanceMatrix.txt",true));
		int noRows = similarityDistanceMatrix2.length;
		int noCols = similarityDistanceMatrix2[0].length;
		
		for(int i = 0; i < noRows; i++){
			for(int j = 0; j < noCols; j++){
				outFile.print(similarityDistanceMatrix2[i][j]+";");
			}
			outFile.println();
		}
		outFile.close();
	}
	

}
