package org.processmining.plugins.actitrac.ui;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.actitrac.ActiTraCClustering;
import org.processmining.plugins.actitrac.algorithm.ProcessModel;
import org.processmining.plugins.actitrac.types.Cluster;


/**
 * This graphical component will display some general information on a given
 * ActiTraC-clustering.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-08-01
 */
public class JClusteringInfo extends JPanel {

	private static final long serialVersionUID = -2617736083471566646L;

	/**
	 * Create a new JClusteringInfo-panel that will display some general
	 * information on the given ActiTraCClustering.
	 */
	public JClusteringInfo(ActiTraCClustering clustering) {
		this.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		this.setOpaque(false);

		ArrayList<Cluster> clusters = new ArrayList<Cluster>();
		for (Cluster c : clustering.getTree())
			clusters.add(c);

		float fitness = 0;
		float nbTraces = 0;
		float pm = 0;
		for (Cluster c : clusters) {
			int traces = c.getExternalLog().size() + c.getInternalLog().size();
			nbTraces += traces;
			fitness += traces * c.getFitness();
			ProcessModel model = c.getProcessModel();
			for (XTrace trace : c.getExternalLog())
				if (model.traceFitness(trace) == 1.0f)
					pm++;
			for (XTrace trace : c.getInternalLog())
				if (model.traceFitness(trace) == 1.0f)
					pm++;
		}
		fitness /= nbTraces;
		pm /= nbTraces;

		this.setLayout(new GridBagLayout());
		GridBagConstraints c1 = new GridBagConstraints();
		GridBagConstraints c2 = new GridBagConstraints();
		c1.gridx = 0;
		c2.gridx = 1;
		c1.anchor = GridBagConstraints.BASELINE_LEADING;
		c2.anchor = GridBagConstraints.BASELINE_LEADING;
		c1.insets = new Insets(0, 0, 0, 20);
		Font font = this.getFont().deriveFont(Font.BOLD, 13f);

		{
			// Average ICS-fitness
			JLabel one = new JLabel("Average ICS-fitness");
			JLabel two = new JLabel(Float.toString(fitness));
			one.setFont(font);
			two.setFont(font);
			c1.gridy = 0;
			c2.gridy = 0;
			this.add(one, c1);
			this.add(two, c2);
		}

		{
			// Average PM
			JLabel one = new JLabel("Average PM");
			JLabel two = new JLabel(Float.toString(pm));
			one.setFont(font);
			two.setFont(font);
			c1.gridy = 1;
			c2.gridy = 1;
			this.add(one, c1);
			this.add(two, c2);
		}

		{
			// Number of clusters
			JLabel one = new JLabel("Number of clusters");
			JLabel two = new JLabel(Integer.toString(clusters.size()));
			one.setFont(font);
			two.setFont(font);
			c1.gridy = 2;
			c2.gridy = 2;
			this.add(one, c1);
			this.add(two, c2);
		}
	}
}
