package org.processmining.plugins.actitrac.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.processmining.plugins.actitrac.utils.ATCParameters;


import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * The first panel of the ActiTraC-preferences-wizard.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-08-01
 */
public class PanelOne extends JPanel {

	private static final long serialVersionUID = -5858631772351410585L;

	private FancyDoubleSlider minimal_size_slider;

	private JRadioButton max_cluster_radio;
	private JRadioButton min_clustered_radio;
	private FancyIntegerSlider max_cluster_slider;
	private FancyDoubleSlider min_clustered_slider;
	private FancyDoubleSlider targetFitness_slider;

	private JRadioButton nonFitting_separate;
	private JRadioButton nonFitting_add;

	private JRadioButton algorithmGreedy;
	private JRadioButton algorithmGreedyDistance;
	private FancyDoubleSlider algorithmGreediness;

	/**
	 * Creates the entire first panel of the settings.
	 */
	public PanelOne() {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setOpaque(false);

		this.add(initAlgorithmPane());
		this.add(Box.createVerticalStrut(10));
		this.add(initClusterPane());
		this.add(Box.createVerticalStrut(10));
		this.add(initStopPane());
		this.add(Box.createVerticalStrut(10));
		this.add(initNonFitting());
		this.add(Box.createVerticalGlue());
	}

	/**
	 * Return the parameters that the current state of the panel represent.
	 */
	public ATCParameters getMDTCParameters() {
		ATCParameters parameters = new ATCParameters();
		parameters.setGreedy(algorithmGreedy.isSelected());
		parameters.setGreedyDistance(algorithmGreedyDistance.isSelected());
		parameters
				.setDistanceGreediness((float) algorithmGreediness.getValue());
		parameters
				.setMinimalClusterSize((float) minimal_size_slider.getValue());
		parameters.setStopByPercentage(min_clustered_radio.isSelected());
		parameters.setStopByNbGroups(max_cluster_radio.isSelected());
		parameters.setSeparateNonFitting(nonFitting_separate.isSelected());
		parameters.setStopNbGroups(max_cluster_slider.getValue());
		parameters.setStopPercentage((float) min_clustered_slider.getValue());
		parameters.setTargetFitness((float) targetFitness_slider.getValue());
		return parameters;
	}

	/**
	 * Create the "Algorithm"-section.
	 */
	private JPanel initAlgorithmPane() {
		SlickerFactory slicker = SlickerFactory.instance();
		JPanel pane = slicker.createRoundedPanel(15, Color.gray);
		pane.setLayout(new GridBagLayout());

		JLabel title = slicker.createLabel("Algorithm");
		title.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		title.setForeground(new Color(40, 40, 40));

		algorithmGreedy = slicker
				.createRadioButton("Frequency-based selective sampling");
		algorithmGreedyDistance = slicker
				.createRadioButton("MRA distance-based selective sampling");

		algorithmGreediness = new FancyDoubleSlider(0, 0.25, 1);
		final JLabel greedinessLabel = slicker.createLabel("Window size");

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(title, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(algorithmGreedy, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(algorithmGreedyDistance, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 3;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(greedinessLabel, c);

		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 3;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(algorithmGreediness, c);

		ButtonGroup group = new ButtonGroup();
		group.add(algorithmGreedy);
		group.add(algorithmGreedyDistance);
		algorithmGreedy.setSelected(true);
		algorithmGreediness.setEnabled(false);
		greedinessLabel.setForeground(algorithmGreedy.getForeground());
		ActionListener l = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				algorithmGreediness.setEnabled(algorithmGreedyDistance
						.isSelected());
			}
		};
		algorithmGreedy.addActionListener(l);
		algorithmGreedyDistance.addActionListener(l);

		return pane;
	}

	/**
	 * Create the "Cluster Characteristics"-section.
	 */
	private JPanel initClusterPane() {
		SlickerFactory slicker = SlickerFactory.instance();
		JPanel pane = slicker.createRoundedPanel(15, Color.GRAY);
		pane.setLayout(new GridBagLayout());

		JLabel title = slicker.createLabel("Cluster Characteristrics");
		title.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		title.setForeground(new Color(40, 40, 40));

		JLabel minimal_size_label = slicker.createLabel("Minimal cluster size");
		minimal_size_label.setForeground(new Color(40, 40, 40));
		minimal_size_slider = new FancyDoubleSlider(0, 0.25, 1);

		JLabel targetFitness_label = slicker.createLabel("Target ICS-fitness");
		targetFitness_label.setForeground(new Color(40, 40, 40));
		targetFitness_slider = new FancyDoubleSlider(0, 1, 1);

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(title, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(minimal_size_label, c);

		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(minimal_size_slider, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(targetFitness_label, c);

		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 2;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.LINE_START;
		pane.add(targetFitness_slider, c);

		pane.setMaximumSize(new Dimension(Integer.MAX_VALUE, pane
				.getPreferredSize().height));
		return pane;
	}

	/**
	 * Create the "Remaining traces"-section.
	 */
	private JPanel initNonFitting() {
		SlickerFactory slicker = SlickerFactory.instance();
		JPanel panel = slicker.createRoundedPanel(15, Color.GRAY);
		panel.setLayout(new GridBagLayout());

		JLabel title = slicker.createLabel("Remaining traces");
		title.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		title.setForeground(new Color(40, 40, 40));

		nonFitting_separate = slicker
				.createRadioButton("Add remaining traces to a new cluster");
		nonFitting_add = slicker
				.createRadioButton("Add remaining traces to other clusters");

		ButtonGroup group = new ButtonGroup();
		group.add(nonFitting_separate);
		group.add(nonFitting_add);
		nonFitting_separate.setSelected(true);

		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.LINE_START;
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		panel.add(title, c);

		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.LINE_START;
		c.gridx = 0;
		c.gridy = 1;
		panel.add(nonFitting_separate, c);

		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.LINE_START;
		c.gridx = 0;
		c.gridy = 2;
		panel.add(nonFitting_add, c);

		panel.setMaximumSize(new Dimension(Integer.MAX_VALUE, panel
				.getPreferredSize().height));

		return panel;
	}

	/**
	 * Create the "Stop criteria"-section.
	 */
	private JPanel initStopPane() {
		SlickerFactory slicker = SlickerFactory.instance();
		JPanel stop_pane = slicker.createRoundedPanel(15, Color.GRAY);
		stop_pane.setLayout(new GridBagLayout());

		JLabel title = slicker.createLabel("Stop criteria");
		title.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		title.setForeground(new Color(40, 40, 40));

		max_cluster_radio = slicker
				.createRadioButton("Maximal amount of clusters");
		min_clustered_slider = new FancyDoubleSlider(0, 90, 100);
		min_clustered_radio = slicker
				.createRadioButton("Minimal percentage of clustered traces");
		max_cluster_slider = new FancyIntegerSlider(2, 4, 100, true);

		ButtonGroup stop_group = new ButtonGroup();
		stop_group.add(max_cluster_radio);
		stop_group.add(min_clustered_radio);

		max_cluster_radio.setSelected(true);
		min_clustered_slider.setEnabled(false);

		ActionListener stop_switcher = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				max_cluster_slider.setEnabled(max_cluster_radio.isSelected());
				min_clustered_slider.setEnabled(min_clustered_radio
						.isSelected());
			}
		};
		max_cluster_radio.addActionListener(stop_switcher);
		min_clustered_radio.addActionListener(stop_switcher);

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		title.setMaximumSize(title.getPreferredSize());
		stop_pane.add(title, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		stop_pane.add(max_cluster_radio, c);

		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 1.0;
		stop_pane.add(max_cluster_slider, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.anchor = GridBagConstraints.LINE_START;
		stop_pane.add(min_clustered_radio, c);

		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 2;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.LINE_START;
		stop_pane.add(min_clustered_slider, c);

		stop_pane.setMaximumSize(new Dimension(Integer.MAX_VALUE, stop_pane
				.getPreferredSize().height));

		return stop_pane;
	}

}
