package org.processmining.plugins.actitrac.ui;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.actitrac.utils.ATCParameters;
import org.processmining.plugins.actitrac.utils.OperationCancelledException;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.gui.ParametersPanel;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.settings.HeuristicsMinerSettings;


/**
 * Represents a wizard that will ask the user for all ActiTraC and
 * HeuristicsMiner-parameters.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-07-29
 */
public class ParametersWizard {

	/**
	 * The UIPluginContext in which the wizard will be shown.
	 */
	private UIPluginContext context;

	/**
	 * The resulting ATCParameters.
	 */
	private ATCParameters parameters;

	/**
	 * The resulting settings for HeuristicsMiner.
	 */
	private HeuristicsMinerSettings heuristicsMinerSettings;

	/**
	 * Create a new ParametersWizard in the given context.
	 */
	public ParametersWizard(UIPluginContext context) {
		this.context = context;
	}

	/**
	 * Get the HeuristicsMiner-settings which the user has entered. The result
	 * of this method is not guaranteed until the {@link #show()} has returned
	 * properly.
	 */
	public HeuristicsMinerSettings getHeuristicsMinerSettings() {
		return heuristicsMinerSettings;
	}

	/**
	 * Get the ActiTraC-parameters which the user has entered. The result of
	 * this method is not guaranteed until the {@link #show()} has returned
	 * properly.
	 */
	public ATCParameters getATCParameters() {
		return parameters;
	}

	/**
	 * Show the wizard in the given context. This method will not return until
	 * the user has either cancelled or selected correct parameters.
	 * 
	 * @throws OperationCancelledException
	 *             When the user cancels the wizard.
	 */
	public void show() throws OperationCancelledException {
		PanelOne panel_one = new PanelOne();
		ParametersPanel panel_two = new ParametersPanel();

		InteractionResult result = InteractionResult.NEXT;
		int step = -1;

		while (true) {
			switch (result) {
			case CANCEL:
				throw new OperationCancelledException();
			case CONTINUE:
				step++;
				break;
			case FINISHED:
				step = 2;
				break;
			case NEXT:
				step++;
				break;
			case PREV:
				step--;
				break;
			}
			switch (step) {
			case -1:
			case 0:
				result = context.showWizard("ActiTraC Parameters", true,
						false, panel_one);
				break;
			case 1:
				result = context.showWizard("Heuristics Miner Parameters",
						false, true, panel_two);
				break;
			case 2:
				parameters = panel_one.getMDTCParameters();
				heuristicsMinerSettings = panel_two.getSettings();
				return;
			}
		}
	}
}
