package org.processmining.plugins.actitrac.utils;

/**
 * This class represents all of the parameters that ActiTraC will take into
 * account.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-07-29
 */
public class ATCParameters {

	/**
	 * The greediness of the algorithm. The fraction of most frequent dpi in
	 * which a different selection can be done.
	 */
	private float distanceGreediness;

	/**
	 * True if the algorithm to be used is the greedy (frequency first) variant.
	 */
	private boolean greedy;

	/**
	 * True if the algorithm to be used is the variant that works using both
	 * greediness and a sense of distance metrics.
	 */
	private boolean greedyDistance;

	/**
	 * The minimal size of a cluster, expressed in as a fraction of the traces
	 * that have not yet been clustered.
	 */
	private float minimalClusterSize;

	/**
	 * True if traces that don't fit in any cluster when the stop criteria have
	 * been reached should be assigned a separate cluster.
	 */
	private boolean separateNonFitting;

	/**
	 * True if the stop criterion is having created a certain amount of
	 * clusters.
	 */
	private boolean stopByNbGroups;

	/**
	 * True if the stop criterion is having clustered a certain percentage of
	 * traces.
	 */
	private boolean stopByPercentage;

	/**
	 * The amount of groups that has to be created. Only valid when
	 * stopByNbGroups is true.
	 */
	private int stopNbGroups;

	/**
	 * The percentage of traces that has to be clustered. Only valid when
	 * stopByPercentage is true.
	 */
	private float stopPercentage;

	/**
	 * The minimal fitness that each cluster must have.
	 */
	private float targetFitness;

	/**
	 * @see #distanceGreediness
	 */
	public float getDistanceGreediness() {
		return distanceGreediness;
	}

	/**
	 * @see #minimalClusterSize
	 */
	public float getMinimalClusterSize() {
		return minimalClusterSize;
	}

	/**
	 * @see #separateNonFitting
	 */
	public boolean getSeparateNonFitting() {
		return separateNonFitting;
	}

	/**
	 * @see #stopByNbGroups
	 */
	public boolean getStopByNbGroups() {
		return stopByNbGroups;
	}

	/**
	 * @see #stopByPercentage
	 */
	public boolean getStopByPercentage() {
		return stopByPercentage;
	}

	/**
	 * @see #stopNbGroups
	 */
	public int getStopNbGroups() {
		return stopNbGroups;
	}

	/**
	 * @see #stopPercentage
	 */
	public float getStopPercentage() {
		return stopPercentage;
	}

	/**
	 * @see #targetFitness
	 */
	public float getTargetFitness() {
		return targetFitness;
	}

	/**
	 * @see #greedy
	 */
	public boolean isGreedy() {
		return greedy;
	}

	/**
	 * @see #greedyDistance
	 */
	public boolean isGreedyDistance() {
		return greedyDistance;
	}

	/**
	 * @see #distanceGreediness
	 */
	public void setDistanceGreediness(float distanceGreediness) {
		this.distanceGreediness = distanceGreediness;
	}

	/**
	 * @see #greedy
	 */
	public void setGreedy(boolean greedy) {
		this.greedy = greedy;
	}

	/**
	 * @see #greedyDistance
	 */
	public void setGreedyDistance(boolean greedyDistance) {
		this.greedyDistance = greedyDistance;
	}

	/**
	 * @see #minimalClusterSize
	 */
	public void setMinimalClusterSize(float minimalClusterSize) {
		this.minimalClusterSize = minimalClusterSize;
	}

	/**
	 * @see #separateNonFitting
	 */
	public void setSeparateNonFitting(boolean separateNonFitting) {
		this.separateNonFitting = separateNonFitting;
	}

	/**
	 * @see #stopByNbGroups
	 */
	public void setStopByNbGroups(boolean stopByNbGroups) {
		this.stopByNbGroups = stopByNbGroups;
	}

	/**
	 * @see #stopByPercentage
	 */
	public void setStopByPercentage(boolean stopByPercentage) {
		this.stopByPercentage = stopByPercentage;
	}

	/**
	 * @see #stopNbGroups
	 */
	public void setStopNbGroups(int stopNbGroups) {
		this.stopNbGroups = stopNbGroups;
	}

	/**
	 * @see #stopPercentage
	 */
	public void setStopPercentage(float stopPercentage) {
		this.stopPercentage = stopPercentage;
	}

	/**
	 * @see #targetFitness
	 */
	public void setTargetFitness(float targetFitness) {
		this.targetFitness = targetFitness;
	}
}
