package org.processmining.plugins.actitrac.utils;

import org.processmining.plugins.actitrac.algorithm.Clusterer;
import org.processmining.plugins.actitrac.types.Cluster;


/**
 * This interface specifies event listeners for use with the {@link Clusterer}
 * -class.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-07-29
 */
public interface ClustererEventListener {

	/**
	 * You should call this method when the underlying Clusterer has cancelled.
	 */
	public void cancel();

	/**
	 * Indicates that a new Cluster has been created by the algorithm.
	 * 
	 * @param c
	 *            The newly created cluster
	 */
	public void clusterCreated(Cluster c);

	/**
	 * Indicates that the processing has finished.
	 */
	public void end();

	/**
	 * This method is called when the clustering algorithm is unable to find a
	 * cluster that satisfies the minimal cluster size without decreasing the
	 * fitness of the cluster.
	 * 
	 * This method should indicate whether the Clusterer should continue
	 * operation after generating this error.
	 * 
	 * @return
	 *         True if the Clusterer is allowed to continue operation, false
	 *         otherwise.
	 */
	public boolean minimalClusterSizeImpossible();

	/**
	 * Indicates that a non-fitting group of traces has been added to an
	 * existing cluster.
	 */
	public void nonfittingAssigned();

	/**
	 * This method should return true when the Clusterer should cancel.
	 */
	public boolean shouldCancel();

	/**
	 * Indicates that the clustering has started. This is roughly equivalent to
	 * having called {@link Clusterer#cluster()}.
	 */
	public void start();

	/**
	 * Indicates that the target stop state has been reached. After this event
	 * has been generated, the remaining groups of traces will still be
	 * processed.
	 * 
	 * @param groups
	 *            The amount of groups of traces that not yet have been added to
	 *            a group.
	 */
	public void targetReached(int groups);

	/**
	 * Indicates that the Clusterer is currently importing data.
	 */
	public void importing();
}
